@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 2013-2018 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright © 2013-2023 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -11,17 +11,18 @@
 
 #ifdef IN_UPDATER
 # import <Cocoa/Cocoa.h>
-# import <Sparkle/SUUpdaterDelegate.h>
+# import <Sparkle/SPUUpdaterDelegate.h>
+# import <Sparkle/SPUStandardUserDriverDelegate.h>
 
 @interface XScreenSaverUpdater : NSObject <NSApplicationDelegate,
-                                           SUUpdaterDelegate>
+                                           SPUUpdaterDelegate,
+                                           SPUStandardUserDriverDelegate>
 {
-  NSTimer *timer;
 }
 @end
 #endif // IN_UPDATER
 
-#define UPDATER_DOMAIN "org.jwz.xscreensaver.updater"
+#define UPDATER_DOMAIN "org.jwz.xscreensaver.XScreenSaverUpdater"
 
 // Strings must match Sparkle/SUConstants.m
 #define SUSUEnableAutomaticChecksKey	"SUEnableAutomaticChecks"
@@ -30,13 +31,20 @@
 #define SUAutomaticallyUpdateDef	NO
 #define SUSendProfileInfoKey		"SUSendProfileInfo"
 #define SUSendProfileInfoDef		YES
-#define SUScheduledCheckIntervalKey	"SUScheduledCheckInterval"
-#define SUScheduledCheckIntervalDef	604800
 #define SULastCheckTimeKey		"SULastCheckTime"
+#define SULastCheckTimeDef		"1992-08-17 19:00:00 +0000"
+
+#define SUScheduledCheckIntervalKey	"SUScheduledCheckInterval"
+#ifdef IN_UPDATER
+# define SUScheduledCheckIntervalDef	86400	// Updater: 1 day
+#else
+# define SUScheduledCheckIntervalDef	604800	// Savers: 2 weeks
+#endif
 
 #define UPDATER_DEFAULTS @{					\
   @SUSUEnableAutomaticChecksKey: @SUSUEnableAutomaticChecksDef,	\
   @SUAutomaticallyUpdateKey:	 @SUAutomaticallyUpdateDef,	\
   @SUSendProfileInfoKey:         @SUSendProfileInfoDef,		\
-  @SUScheduledCheckIntervalKey:  @SUScheduledCheckIntervalDef	\
+  @SULastCheckTimeKey:		 @SULastCheckTimeDef,		\
+  @SUScheduledCheckIntervalKey:  @SUScheduledCheckIntervalDef,	\
 }
