@@ -1,5 +1,4 @@
-/* xscreensaver-command, Copyright (c) 1991-1998
- *  by Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver-command, Copyright © 1991-2021 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -14,7 +13,7 @@
 #define _XSCREENSAVER_REMOTE_H_
 
 extern int xscreensaver_command (Display *dpy, Atom command, long arg,
-				 Bool verbose_p);
+				 Bool verbose_p, char **error_ret);
 
 extern void server_xscreensaver_version (Display *dpy,
 					 char **version_ret,
