@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-# Copyright © 2008-2022 Jamie Zawinski <jwz@jwz.org>
+# Copyright © 2008-2025 Jamie Zawinski <jwz@jwz.org>
 #
 # Permission to use, copy, modify, distribute, and sell this software and its
 # documentation for any purpose is hereby granted without fee, provided that
@@ -21,7 +21,7 @@ use diagnostics;
 use strict;
 
 my $progname = $0; $progname =~ s@.*/@@g;
-my ($version) = ('$Revision: 1.39 $' =~ m/\s(\d[.\d]+)\s/s);
+my ($version) = ('$Revision: 1.49 $' =~ m/\s(\d[.\d]+)\s/s);
 
 my $verbose = 0;
 my $debug_p = 0;
@@ -30,29 +30,29 @@ my $debug_p = 0;
 my $text_default_opts = '';
 foreach (qw(text-mode text-literal text-file text-url text-program)) {
   my $s = $_; $s =~ s/-(.)/\U$1/g; $s =~ s/url/URL/si;
-  $text_default_opts .= "{\"-$_\", \".$s\", XrmoptionSepArg, 0},\n";
+  $text_default_opts .= "{\"--$_\", \".$s\", XrmoptionSepArg, 0},\n";
 }
 my $image_default_opts = '';
 foreach (qw(choose-random-images grab-desktop-images)) {
   my $s = $_; $s =~ s/-(.)/\U$1/g;
-  $image_default_opts .= "{\"-$_\", \".$s\", XrmoptionSepArg, 0},\n";
+  $image_default_opts .= "{\"--$_\", \".$s\", XrmoptionSepArg, 0},\n";
 }
 my $xlockmore_default_opts = '';
 foreach (qw(count cycles delay ncolors size font)) {
-  $xlockmore_default_opts .= "{\"-$_\", \".$_\", XrmoptionSepArg, 0},\n";
+  $xlockmore_default_opts .= "{\"--$_\", \".$_\", XrmoptionSepArg, 0},\n";
 }
 $xlockmore_default_opts .= 
- "{\"-wireframe\", \".wireframe\", XrmoptionNoArg, \"true\"},\n" .
- "{\"-3d\", \".use3d\", XrmoptionNoArg, \"true\"},\n" .
- "{\"-no-3d\", \".use3d\", XrmoptionNoArg, \"false\"},\n";
+ "{\"--wireframe\", \".wireframe\", XrmoptionNoArg, \"true\"},\n" .
+ "{\"--3d\", \".use3d\", XrmoptionNoArg, \"true\"},\n" .
+ "{\"--no-3d\", \".use3d\", XrmoptionNoArg, \"false\"},\n";
 
 my $thread_default_opts = 
-  "{\"-threads\",    \".useThreads\", XrmoptionNoArg, \"True\"},\n" .
-  "{\"-no-threads\", \".useThreads\", XrmoptionNoArg, \"False\"},\n";
+  "{\"--threads\",    \".useThreads\", XrmoptionNoArg, \"True\"},\n" .
+  "{\"--no-threads\", \".useThreads\", XrmoptionNoArg, \"False\"},\n";
 
 my $analogtv_default_opts = '';
 foreach (qw(color tint brightness contrast)) {
-  $analogtv_default_opts .= "{\"-tv-$_\", \".TV$_\", XrmoptionSepArg, 0},\n";
+  $analogtv_default_opts .= "{\"--tv-$_\", \".TV$_\", XrmoptionSepArg, 0},\n";
 }
 
 $analogtv_default_opts .= $thread_default_opts;
@@ -61,7 +61,7 @@ $analogtv_default_opts .= $thread_default_opts;
 
 # Returns two tables:
 # - A table of the default resource values.
-# - A table of "-switch" => "resource: value", or "-switch" => "resource: %"
+# - A table of "--switch" => "resource: value", or "--switch" => "resource: %"
 #
 sub parse_src($) {
   my ($saver) = @_;
@@ -155,11 +155,11 @@ sub parse_src($) {
   print STDERR "$progname: $file: switches to resources:\n"
     if ($verbose > 2);
   my %switch_to_res;
-  $switch_to_res{'-fps'} = 'doFPS: true';
-  $switch_to_res{'-fg'}  = 'foreground: %';
-  $switch_to_res{'-bg'}  = 'background: %';
-  $switch_to_res{'-no-grab-desktop-images'}  = 'grabDesktopImages: false';
-  $switch_to_res{'-no-choose-random-images'}  = 'chooseRandomImages: false';
+  $switch_to_res{'--fps'} = 'doFPS: true';
+  $switch_to_res{'--fg'}  = 'foreground: %';
+  $switch_to_res{'--bg'}  = 'background: %';
+  $switch_to_res{'--no-grab-desktop-images'}  = 'grabDesktopImages: false';
+  $switch_to_res{'--no-choose-random-images'}  = 'chooseRandomImages: false';
 
   my ($ign, $opts) = ($body =~ m/(_options|\bopts)\s*\[\]\s*=\s*{(.*?)}\s*;/s);
   if  ($xlockmore_p || $thread_p || $analogtv_p || $opts) {
@@ -180,12 +180,13 @@ sub parse_src($) {
                   \s * \"([^\"]+)\" \s* ,
                   \s * ([^\s]+)     \s* ,
                   \s * (\"([^\"]*)\"|([a-zA-Z\d_]+)) \s* }@xi;
+      $switch =~ s/^(-[^-])/-$1/s;  # "-foo" => "--foo"
       print STDERR "$progname: $file: unparsable: $_\n" unless $switch;
       my $val = defined($v1) ? $v1 : $v2;
       $val = '%' if ($type eq 'XrmoptionSepArg');
       $res =~ s/^[.*]//s;
       $res =~ s/^[a-z\d]+\.//si;
-      $switch =~ s/^\+/-no-/s;
+      $switch =~ s/^\+/--no-/s;
 
       $val = "$res: $val";
       if (defined ($switch_to_res{$switch})) {
@@ -232,19 +233,19 @@ sub parse_xml($$$) {
     <select id="textMode">
       <option id="date"  _label="Display the date and time"/>
       <option id="text"  _label="Display static text"
-        arg-set="-text-mode literal"/>
+        arg-set="--text-mode literal"/>
       <option id="url"   _label="Display the contents of a URL"
-        arg-set="-text-mode url"/>
+        arg-set="--text-mode url"/>
     </select>
-    <string id="textLiteral" _label="Text to display" arg="-text-literal %"/>
-    <string id="textURL" _label="URL to display" arg="-text-url %"/>
+    <string id="textLiteral" _label="Text to display" arg="--text-literal %"/>
+    <string id="textURL" _label="URL to display" arg="--text-url %"/>
     @gs;
 
   $body =~ s@<xscreensaver-image\s*/?>@
     <boolean id="grabDesktopImages" _label="Grab screenshots"
-       arg-unset="-no-grab-desktop-images"/>
+       arg-unset="--no-grab-desktop-images"/>
     <boolean id="chooseRandomImages" _label="Use photo library"
-       arg-unset="-no-choose-random-images"/>
+       arg-unset="--no-choose-random-images"/>
     @gs;
 
   $body =~ s/<!--.*?-->/ /gsi;
@@ -386,6 +387,7 @@ sub parse_xml($$$) {
         my ($set)   = $ctrl->{'arg-set'};
         my ($unset) = $ctrl->{'arg-unset'};
         my ($arg) = $set || $unset || error ("$file: unparsable: $args");
+        $arg = lc($arg);
         my ($res) = $switch_to_res->{$arg};
           error ("$file: no resource for boolean switch \"$arg\"") unless $res;
 
@@ -485,8 +487,8 @@ sub check_config($$) {
     $progclass = 'DNAlogo' if ($progclass eq 'DNALogo');
     my $f = (glob("$obd/$progclass.saver*"))[0];
     if (!$f && $progclass ne 'Flurry') {
-      print STDERR "$progname: $progclass.saver does not exist\n";
-      $failures++;
+      print STDERR "$progname: WARNING: $progclass.saver does not exist\n";
+      # $failures++;
     }
   }
 
@@ -669,21 +671,25 @@ sub munge_blurb($$$$) {
     }
   }
 
+  my $desc0 = "$desc";
+  utf8::decode($desc0);   # Pack UTF-8 into wide chars.
+
   my $desc1 = ("$name, version $vers.\n\n" .		# savername.xml
                $desc . "\n" .
                "\n" . 
                "From the XScreenSaver collection: " .
                "https://www.jwz.org/xscreensaver/\n" .
-               "Copyright \302\251 $year by $authors.\n");
+               "Copyright \x{A9} $year by $authors.\n");
 
   my $desc2 = ("$name $vers,\n" .			# Info.plist
-               "\302\251 $year $authors.\n" .
+               "\x{A9} $year $authors.\n" .
                #"From the XScreenSaver collection:\n" .
                #"https://www.jwz.org/xscreensaver/\n" .
                "\n" .
                $desc .
                "\n");
-  utf8::decode($desc1);   # Pack UTF-8 into wide chars.
+
+  utf8::decode($desc1);   # Pack UTF-8 into wide chars (redundant?)
   utf8::decode($desc2);
 
   # unwrap lines, but only when it's obviously ok: leave blank lines,
@@ -750,6 +756,16 @@ sub build_android(@) {
     $saver_class =~ s/\]\[/2/gs;
     $saver_class =~ s/[-_\s]//gs;
 
+    # If the saver's title is not case-insensitively the same as its progname
+    # after removing spaces, the class name has to be the progname, or
+    # jwxyz_nativeInit is not able to find it in the function_table.
+    # (Cuboctahedron Eversion, Möbius, MöbiusGears, Moiré, Moiré2.)
+    #
+    if (lc($saver_class) ne lc($saver)) {
+      $saver_class = $saver;
+      $saver_class =~ s/^(.)/\U$1/s;
+    }
+
     my $settings = '';
 
     my $localize0 = sub($$) {
@@ -991,6 +1007,7 @@ sub build_android(@) {
 
     my $dream = ("<dream xmlns:android=\"" .
                    "http://schemas.android.com/apk/res/android\"\n" .
+                 "  android:previewImage=\"\@drawable/$saver_underscore\"\n" .
                  "  android:settingsActivity=\"" .
                      "$package.gen.Settings\$$saver_class\" />\n");
     $write_files{"$xml_dir/${saver_underscore}_dream.xml"} = $dream;
@@ -999,7 +1016,7 @@ sub build_android(@) {
                        "http://schemas.android.com/apk/res/android\"\n" .
                      "  android:settingsActivity=\"" .
                      "$package.gen.Settings\$$saver_class\"\n" .
-                     "  android:thumbnail=\"\@drawable/${saver_underscore}\" />\n");
+                     "  android:thumbnail=\"\@drawable/$saver_underscore\" />\n");
     $write_files{"$xml_dir/${saver_underscore}_wallpaper.xml"} = $wallpaper;
 
     $daydream_java .=
@@ -1067,12 +1084,19 @@ sub build_android(@) {
                 "  android:theme=\"\@android:style/Theme.Holo\"\n" .
                 "  android:exported=\"true\"\n" .
                 "  android:label=\"\@string/app_name\">\n" .
-                "  <intent-filter>\n" .
-                "    <action android:name=\"android.intent.action" .
-                ".MAIN\" />\n" .
-                "    <category android:name=\"android.intent.category" .
-                ".LEANBACK_LAUNCHER\" />\n" .
-                "  </intent-filter>\n" .
+
+                # I have no idea what this error means, when uploading to
+                # the Play store: "app must be published as an app bundle
+                # instead". I also don't know what leanback means or why
+                # I should care, so fuck it.
+                #
+#               "  <intent-filter>\n" .
+#               "    <action android:name=\"android.intent.action" .
+#               ".MAIN\" />\n" .
+#               "    <category android:name=\"android.intent.category" .
+#               ".LEANBACK_LAUNCHER\" />\n" .
+#               "  </intent-filter>\n" .
+
                 "  <intent-filter>\n" .
                 "    <action android:name=\"android.intent.action" .
                 ".VIEW\" />\n" .
@@ -1095,7 +1119,7 @@ sub build_android(@) {
   $manifest = ($xml_header .
                "<manifest xmlns:android=\"" .
                "http://schemas.android.com/apk/res/android\"\n" .
-               "  package=\"$package\"\n" .
+#              "  package=\"$package\"\n" .
                "  android:versionCode=\"$versb\"\n" .
                "  android:versionName=\"$vers\">\n" .
 
@@ -1115,8 +1139,9 @@ sub build_android(@) {
                "  <uses-feature android:glEsVersion=\"0x00010001\"\n" .
                "    android:required=\"true\" />\n" .
 
-               "  <uses-feature android:name=\"android.software.leanback\"\n" .
-               "    android:required=\"false\" />\n" .
+               # See above
+#              "  <uses-feature android:name=\"android.software.leanback\"\n" .
+#              "    android:required=\"false\" />\n" .
 
                "  <uses-feature" .
                " android:name=\"android.hardware.touchscreen\"\n" .
@@ -1126,6 +1151,8 @@ sub build_android(@) {
                    "android.permission.INTERNET\" />\n" .
                "  <uses-permission android:name=\"" .
                    "android.permission.READ_EXTERNAL_STORAGE\" />\n" .
+               "  <uses-permission android:name=\"" .
+                   "android.permission.READ_MEDIA_IMAGES\" />\n" .
 
                "  <application android:icon=\"\@drawable/thumbnail\"\n" .
                "    android:banner=\"\@drawable/thumbnail\"\n" .
@@ -1228,7 +1255,7 @@ sub error($) {
 }
 
 sub usage() {
-  print STDERR "usage: $progname [--verbose] [--debug]" .
+  print STDERR "usage: $progname [--verbose] [--debug] [--force]" .
     " [--build-android] files ...\n";
   exit 1;
 }
@@ -1237,11 +1264,13 @@ sub main() {
   binmode (STDOUT, ':utf8');
   binmode (STDERR, ':utf8');
 
+  my $force_p = 0;
   my $android_p = 0;
   my @files = ();
   while ($#ARGV >= 0) {
     $_ = shift @ARGV;
     if (m/^--?verbose$/) { $verbose++; }
+    elsif (m/^--?force$/) { $force_p++; }
     elsif (m/^-v+$/) { $verbose += length($_)-1; }
     elsif (m/^--?debug$/s) { $debug_p++; }
     elsif (m/^--?build-android$/s) { $android_p++; }
@@ -1250,7 +1279,7 @@ sub main() {
 #    else { usage; }
   }
 
-  usage unless ($#files >= 0);
+  usage unless ($#files >= 0 || $force_p);
   my $failures = 0;
   foreach my $file (@files) {
     $failures += check_config ($file, $android_p);
