@@ -21,8 +21,6 @@
 			"*wireframe:    False         \n" \
 
 # define release_matrix 0
-#undef countof
-#define countof(x) (sizeof((x))/sizeof((*x)))
 
 #undef BELLRAND
 #define BELLRAND(n) ((frand((n)) + frand((n)) + frand((n))) / 3)
@@ -251,7 +249,7 @@ reset_strip (ModeInfo *mi, strip *s)
     if (do_clock &&
         !time_displayed_p &&
         (i < GRID_SIZE-5) &&   /* display approx. once per 5 strips */
-	!(random() % (GRID_SIZE-5)*5))
+	!(random() % ((GRID_SIZE-5)*5)))
       {
 	int j;
 	char text[80];
