@@ -1,10 +1,11 @@
-/* glslideshow, Copyright (c) 2003-2014 Jamie Zawinski <jwz@jwz.org>
+/* glslideshow, Copyright © 2003-2022 Jamie Zawinski <jwz@jwz.org>
  * Loads a sequence of images and smoothly pans around them; crossfades
  * when loading new images.
  *
  * Originally written by Mike Oliphant <oliphant@gtk.org> (c) 2002, 2003.
  * Rewritten by jwz, 21-Jun-2003.
  * Rewritten by jwz again, 6-Feb-2005.
+ * Modified by Richard Weeks <rtweeks21@gmail.com> Copyright (c) 2020
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -16,14 +17,6 @@
  *
  *****************************************************************************
  *
- * TODO:
- *
- * - When a new image is loaded, there is a glitch: animation pauses during
- *   the period when we're loading the image-to-fade-in.  On fast (2GHz)
- *   machines, this stutter is short but noticable (usually around 1/10th
- *   second.)  On slower machines, it can be much more pronounced.
- *   This turns out to be hard to fix...
- *
  *   Image loading happens in three stages:
  *
  *    1: Fork a process and run xscreensaver-getimage in the background.
@@ -34,14 +27,11 @@
  *       (or XShmGetImage.)
  *
  *    3: Once we have the bits, we must convert them from server-native bitmap
- *       layout to 32 bit RGBA in client-endianness, to make them usable as
- *       OpenGL textures.
+ *       layout to 32 bit RGBA in client-endianness, and copy that into an
+ *       OpenGL texture.
  *
- *    4: We must actually construct a texture.
- *
- *   So, the speed of step 1 doesn't really matter, since that happens in
- *   the background.  But steps 2, 3, and 4 happen in *this* process, and
- *   cause the visible glitch.
+ *   The speed of step 1 doesn't really matter, since that happens in the
+ *   background.  Steps 2 and 3 happen in *this* process.
  *
  *   Step 2 can't be moved to another process without opening a second
  *   connection to the X server, which is pretty heavy-weight.  (That would
@@ -49,23 +39,13 @@
  *   retrieve the pixmap, and feed it back to us through a pipe or
  *   something.)
  *
- *   Step 3 might be able to be optimized by coding tuned versions of
- *   grab-ximage.c:copy_ximage() for the most common depths and bit orders.
- *   (Or by moving it into the other process along with step 2.)
+ *   Step 3 is carried out over the course of several animation frames.  The
+ *   bits are processed in "stripes" small enough to complete within a single
+ *   frame.  Each stripe is first converted to a client-endian, 32 bit RGBA
+ *   XImage, then copied into the partially completed OpenGL texture.
  *
- *   Step 4 is the hard one, though.  It might be possible to speed up this
- *   step if there is some way to allow two GL processes share texture
- *   data.  Unless, of course, all the time being consumed by step 4 is
- *   because the graphics pipeline is flooded, in which case, that other
- *   process would starve the screen anyway.
- *
- *   Is it possible to use a single GLX context in a multithreaded way?
- *   Or use a second GLX context, but allow the two contexts to share data?
- *   I can't find any documentation about this.
- *
- *   How does Apple do this with their MacOSX slideshow screen saver?
- *   Perhaps it's easier for them because their OpenGL libraries have
- *   thread support at a lower level?
+ *   This entire process is accomplished via a texture loader created with
+ *   alloc_texture_loader.
  */
 
 #define DEFAULTS  "*delay:           20000                \n" \
@@ -73,7 +53,7 @@
                   "*showFPS:         False                \n" \
 	          "*fpsSolid:        True                 \n" \
 	          "*useSHM:          True                 \n" \
-            "*titleFont: -*-helvetica-medium-r-normal-*-*-180-*-*-*-*-*-*\n" \
+                  "*titleFont: sans-serif 18\n" \
                   "*desktopGrabber:  xscreensaver-getimage -no-desktop %s\n" \
 		  "*grabDesktopImages:   False \n" \
 		  "*chooseRandomImages:  True  \n"
@@ -81,9 +61,6 @@
 # define release_slideshow 0
 # include "xlockmore.h"
 
-#undef countof
-#define countof(x) (sizeof((x))/sizeof((*x)))
-
 #ifdef USE_GL
 
 
@@ -95,11 +72,16 @@
 # define DEF_TITLES         "False"
 # define DEF_LETTERBOX      "True"
 # define DEF_DEBUG          "False"
+# define DEF_VERBOSE        "False"
 # define DEF_MIPMAP         "True"
 
 #include "grab-ximage.h"
 #include "texfont.h"
 
+# ifndef HAVE_JWXYZ
+#  include <X11/Intrinsic.h>     /* for XrmDatabase in -verbose mode */
+# endif
+
 typedef struct {
   double x, y, w, h;
 } rect;
@@ -116,6 +98,8 @@ typedef struct {
                                       on screen */
   GLuint texid;			   /* which texture contains the image */
   int refcount;			   /* how many sprites refer to this image */
+  texture_loader_t *loader;	   /* asynchronous image loader */
+  int steps;			   /* How many steps it took to load */
 } image;
 
 
@@ -189,7 +173,8 @@ static Bool letterbox_p;    /* When a loaded image is not the same aspect
                              */
 static Bool mipmap_p;	    /* Use mipmaps instead of single textures. */
 static Bool do_titles;	    /* Display image titles. */
-static Bool debug_p;	    /* Be loud and do weird things. */
+static Bool verbose_p;	    /* Print to stderr. */
+static Bool debug_p;	    /* Show image extents with boxes. */
 
 
 static XrmOptionDescRec opts[] = {
@@ -204,6 +189,8 @@ static XrmOptionDescRec opts[] = {
   {"-clip",         ".letterbox",     XrmoptionNoArg, "False" },
   {"-mipmaps",      ".mipmap",        XrmoptionNoArg, "True"  },
   {"-no-mipmaps",   ".mipmap",        XrmoptionNoArg, "False" },
+  {"-v",            ".verbose",       XrmoptionNoArg, "True"  },
+  {"-verbose",      ".verbose",       XrmoptionNoArg, "True"  },
   {"-debug",        ".debug",         XrmoptionNoArg, "True"  },
 };
 
@@ -215,6 +202,7 @@ static argtype vars[] = {
   { &mipmap_p,      "mipmap",       "Mipmap",       DEF_MIPMAP,        t_Bool},
   { &letterbox_p,   "letterbox",    "Letterbox",    DEF_LETTERBOX,     t_Bool},
   { &fps_cutoff,    "FPScutoff",    "FPSCutoff",    DEF_FPS_CUTOFF,     t_Int},
+  { &verbose_p,     "verbose",      "Verbose",      DEF_VERBOSE,       t_Bool},
   { &debug_p,       "debug",        "Debug",        DEF_DEBUG,         t_Bool},
   { &do_titles,     "titles",       "Titles",       DEF_TITLES,        t_Bool},
 };
@@ -298,6 +286,45 @@ alloc_image (ModeInfo *mi)
 }
 
 
+/* Allocate an image structure and start asynchronous file loading in the
+   background.
+
+   The texture_loader_t referenced by *result->loader must be stepped with
+   step_texture_loader() until it calls back to the callback function passed
+   to it.
+ */
+static image *
+alloc_image_incremental (ModeInfo *mi)
+{
+  slideshow_state *ss = &sss[MI_SCREEN(mi)];
+  int wire = MI_IS_WIREFRAME(mi);
+  image *img = (image *) calloc (1, sizeof (*img));
+
+  img->id = ++ss->image_id;
+  img->loaded_p = False;
+  img->used_p = False;
+  img->mi = mi;
+  img->steps = 0;
+
+  glGenTextures (1, &img->texid);
+  if (img->texid <= 0) abort();
+
+  ss->image_load_time = ss->now;
+
+  if (wire)
+    image_loaded_cb (0, 0, 0, 0, 0, 0, img);
+  else
+    img->loader = alloc_texture_loader (mi->xgwa.screen, mi->window,
+                                        *ss->glx_context, 0, 0, mipmap_p,
+                                        img->texid);
+
+  ss->images[ss->nimages++] = img;
+  if (ss->nimages >= countof(ss->images)) abort();
+
+  return img;
+}
+
+
 /* Callback that tells us that the texture has been loaded.
  */
 static void
@@ -321,6 +348,13 @@ image_loaded_cb (const char *filename, XRectangle *geom,
       goto DONE;
     }
 
+  if (img->loader)
+    {
+      texture_loader_t *loader = img->loader;
+      img->loader = 0;
+      free_texture_loader(loader);
+    }
+
   if (image_width == 0 || image_height == 0)
     exit (1);
 
@@ -361,16 +395,16 @@ image_loaded_cb (const char *filename, XRectangle *geom,
   */
   if (img->title && img->title[0] == '/')
     {
-      /* strip filename to part between last "/" and last ".". */
+      /* strip filename to part between last "/" and end. */
+      /* xscreensaver-getimage has already stripped off the extension. */
       char *s = strrchr (img->title, '/');
-      if (s) strcpy (img->title, s+1);
-      s = strrchr (img->title, '.');
-      if (s) *s = 0;
+      if (s) memmove (img->title, s+1, strlen (s));
     }
 
-  if (debug_p)
-    fprintf (stderr, "%s: loaded   img %2d: \"%s\"\n",
-             blurb(), img->id, (img->title ? img->title : "(null)"));
+  if (verbose_p)
+    fprintf (stderr, "%s: loaded   img %2d: \"%s\" (%d steps)\n",
+             blurb(), img->id, (img->title ? img->title : "(null)"),
+             img->steps + 1);
  DONE:
 
   img->loaded_p = True;
@@ -407,7 +441,7 @@ destroy_image (ModeInfo *mi, image *img)
 
   if (!freed_p) abort();
 
-  if (debug_p)
+  if (verbose_p)
     fprintf (stderr, "%s: unloaded img %2d: \"%s\"\n",
              blurb(), img->id, (img->title ? img->title : "(null)"));
 
@@ -457,7 +491,7 @@ get_image (ModeInfo *mi)
   /* Make sure that there is always one unused image in the pipe.
    */
   if (!new_img && !loading_img)
-    alloc_image (mi);
+    alloc_image_incremental (mi);
 
   return img;
 }
@@ -687,9 +721,9 @@ tick_sprite (ModeInfo *mi, sprite *sp)
     {
       double secs = now - sp->state_time;
 
-      if (debug_p)
+      if (verbose_p)
         fprintf (stderr,
-                 "%s: %s %3d frames %2.0f sec %5.1f fps (%.1f fps?)\n",
+                 "%s: %s %3d frames %2.0f sec %5.1f fps (max %.1f fps?)\n",
                  blurb(),
                  (sp->prev_state == IN ? "fade" : "pan "),
                  sp->frame_count,
@@ -717,7 +751,7 @@ tick_sprite (ModeInfo *mi, sprite *sp)
    its creation time compared to the current wall clock.
  */
 static void
-draw_sprite (ModeInfo *mi, sprite *sp)
+draw_sprite (ModeInfo *mi, sprite *sp, Bool keep_title_p)
 {
   slideshow_state *ss = &sss[MI_SCREEN(mi)];
   int wire = MI_IS_WIREFRAME(mi);
@@ -773,6 +807,7 @@ draw_sprite (ModeInfo *mi, sprite *sp)
         GLfloat texx2 = texx1 + texw;
         GLfloat texy2 = texy1 + texh;
 
+        glEnable (GL_TEXTURE_2D);
         glBindTexture (GL_TEXTURE_2D, img->texid);
         glColor4f (1, 1, 1, sp->opacity);
         glNormal3f (0, 0, 1);
@@ -804,11 +839,9 @@ draw_sprite (ModeInfo *mi, sprite *sp)
       }
 
 
-    if (do_titles &&
-        img->title && *img->title &&
-        (sp->state == IN || sp->state == FULL))
+    if (do_titles && img->title && *img->title)
       {
-        glColor4f (1, 1, 1, sp->opacity);
+        glColor4f (1, 1, 1, keep_title_p ? 1 : sp->opacity);
         print_texture_label (mi->dpy, ss->font_data,
                              mi->xgwa.width, mi->xgwa.height,
                              1, img->title);
@@ -860,6 +893,7 @@ static void
 draw_sprites (ModeInfo *mi)
 {
   slideshow_state *ss = &sss[MI_SCREEN(mi)];
+  char *last_title = 0;
   int i;
 
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
@@ -882,7 +916,16 @@ draw_sprites (ModeInfo *mi)
 */
 
   for (i = 0; i < ss->nsprites; i++)
-    draw_sprite (mi, ss->sprites[i]);
+    {
+      sprite *s = ss->sprites[i];
+      /* If the two sprites have the same title, draw the second title at
+         full intensity to avoid a weird blooming effect when crossfading
+         an image with itself. */
+      Bool keep_title_p = (do_titles && last_title && s->img->title &&
+                           !strcmp (last_title, s->img->title));
+      draw_sprite (mi, s, keep_title_p);
+      last_title = s->img->title;
+    }
   glPopMatrix();
 
   if (debug_p)				/* draw a white box (the "screen") */
@@ -939,11 +982,11 @@ slideshow_handle_event (ModeInfo *mi, XEvent *event)
   slideshow_state *ss = &sss[MI_SCREEN(mi)];
 
   if (event->xany.type == Expose ||
-           event->xany.type == GraphicsExpose ||
-           event->xany.type == VisibilityNotify)
+      event->xany.type == GraphicsExpose ||
+      event->xany.type == VisibilityNotify)
     {
       ss->redisplay_needed_p = True;
-      if (debug_p)
+      if (verbose_p)
         fprintf (stderr, "%s: exposure\n", blurb());
       return False;
     }
@@ -1021,7 +1064,7 @@ check_fps (ModeInfo *mi)
 
   if (fps >= fps_cutoff)
     {
-      if (debug_p)
+      if (verbose_p)
         fprintf (stderr,
                  "%s: %.1f fps is fast enough (with %d frames in %.1f secs)\n",
                  blurb(), fps, ss->frames_elapsed, wall_elapsed);
@@ -1051,24 +1094,25 @@ check_fps (ModeInfo *mi)
 }
 
 
-/* Kludge to add "-v" to invocation of "xscreensaver-getimage" in -debug mode
+/* Add "-v" to invocation of "xscreensaver-getimage" in -debug mode.
  */
 static void
-hack_resources (void)
+hack_resources (Display *dpy)
 {
-#if 0
+# ifndef HAVE_JWXYZ
   char *res = "desktopGrabber";
-  char *val = get_string_resource (res, "DesktopGrabber");
+  char *val = get_string_resource (dpy, res, "DesktopGrabber");
   char buf1[255];
   char buf2[255];
   XrmValue value;
-  sprintf (buf1, "%.100s.%.100s", progclass, res);
+  XrmDatabase db = XtDatabase (dpy);
+  sprintf (buf1, "%.100s.%.100s", progname, res);
   sprintf (buf2, "%.200s -v", val);
   value.addr = buf2;
   value.size = strlen(buf2);
   XrmPutResource (&db, buf1, "String", &value);
   free (val);
-#endif
+# endif /* !HAVE_JWXYZ */
 }
 
 
@@ -1088,13 +1132,15 @@ init_slideshow (ModeInfo *mi)
     MI_CLEARWINDOW(mi);
   }
 
-  if (debug_p)
+  if (debug_p) verbose_p = True;
+
+  if (verbose_p)
     fprintf (stderr, "%s: pan: %d; fade: %d; img: %d; zoom: %d%%\n",
              blurb(), pan_seconds, fade_seconds, image_seconds, zoom);
 
   sanity_check(mi);
 
-  if (debug_p)
+  if (verbose_p)
     fprintf (stderr, "%s: pan: %d; fade: %d; img: %d; zoom: %d%%\n\n",
              blurb(), pan_seconds, fade_seconds, image_seconds, zoom);
 
@@ -1117,7 +1163,7 @@ init_slideshow (ModeInfo *mi)
   ss->font_data = load_texture_font (mi->dpy, "titleFont");
 
   if (debug_p)
-    hack_resources();
+    hack_resources (MI_DISPLAY(mi));
 
   ss->now = double_time();
   ss->dawn_of_time = ss->now;
@@ -1128,6 +1174,9 @@ init_slideshow (ModeInfo *mi)
 }
 
 
+static void slideshow_idle (ModeInfo *mi);
+
+
 ENTRYPOINT void
 draw_slideshow (ModeInfo *mi)
 {
@@ -1204,9 +1253,12 @@ draw_slideshow (ModeInfo *mi)
        frame buffers. Note that this means that the FPS display will be
        wrong: "Load" will be frozen on whatever it last was, when in
        reality it will be close to 0. */
-    return;
+    {
+      slideshow_idle (mi);
+      return;
+    }
 
-  if (debug_p && ss->now - ss->prev_frame_time > 1)
+  if (verbose_p && ss->now - ss->prev_frame_time > 1)
     fprintf (stderr, "%s: static screen for %.1f secs\n",
              blurb(), ss->now - ss->prev_frame_time);
 
@@ -1218,6 +1270,9 @@ draw_slideshow (ModeInfo *mi)
   glXSwapBuffers (MI_DISPLAY (mi), MI_WINDOW(mi));
   ss->prev_frame_time = ss->now;
   ss->redisplay_needed_p = False;
+
+  slideshow_idle (mi);
+
   check_fps (mi);
 }
 
@@ -1249,6 +1304,29 @@ free_slideshow (ModeInfo *mi)
 # endif
 }
 
+
+static void
+slideshow_idle (ModeInfo *mi)
+{
+  slideshow_state *ss = &sss[MI_SCREEN(mi)];
+  double allowed_time = ((double) mi->pause) / 2000000; /* 0.01 sec */
+  int i;
+
+  for (i = 0; i < ss->nimages; i++)
+    {
+      image *img = ss->images[i];
+      if (img->loader)
+        {
+          if (texture_loader_failed (img->loader))
+            abort();
+          step_texture_loader (img->loader, allowed_time,
+                               image_loaded_cb, img);
+          img->steps++;
+          break; /* only do the first one! */
+        }
+    }
+}
+
 XSCREENSAVER_MODULE_2 ("GLSlideshow", glslideshow, slideshow)
 
 #endif /* USE_GL */
