@@ -8,15 +8,7 @@
 #if 0   /* for Mesa */
 # include "glutint.h"
 #else   /* for xscreensaver */
-# ifdef HAVE_CONFIG_H
-#  include "config.h"
-# endif
-# ifndef HAVE_COCOA
-#  include <GL/gl.h>
-# endif
-# ifdef HAVE_JWZGLES
-#  include "jwzgles.h"
-# endif /* HAVE_JWZGLES */
+# include "screenhackI.h"
 # undef APIENTRY
 # define APIENTRY /**/
 #endif
