@@ -245,14 +245,22 @@ setup_display (struct state *st)
     XWindowAttributes xgwa;
 
     int cell_size = get_integer_resource (st->dpy, "size", "Integer");
-    int osize, alloc_size, oalloc;
+    int osize, alloc_size;
+#if 0
+    int oalloc;
+#endif
     int mem_throttle = 0;
     char *s;
 
+    XGetWindowAttributes (st->dpy, st->window, &xgwa);
+
     if (cell_size < 1) cell_size = 1;
 
     osize = cell_size;
 
+    if (xgwa.width > 2560 || xgwa.height > 2560)
+      cell_size *= 2;  /* Retina displays */
+
     s = get_string_resource (st->dpy, "memThrottle", "MemThrottle");
     if (s)
       {
@@ -276,8 +284,6 @@ setup_display (struct state *st)
         free (s);
       }
 
-    XGetWindowAttributes (st->dpy, st->window, &xgwa);
-
     st->originalcolors = get_boolean_resource (st->dpy, "originalcolors", "Boolean");
 
     st->count = get_integer_resource (st->dpy, "count", "Integer");
@@ -404,7 +410,9 @@ setup_display (struct state *st)
     st->arr_height = st->windowHeight / cell_size;
 
     alloc_size = sizeof(cell) * st->arr_width * st->arr_height;
+# if 0
     oalloc = alloc_size;
+# endif
 
     if (mem_throttle > 0)
       while (cell_size < st->windowWidth/10 &&
@@ -419,7 +427,8 @@ setup_display (struct state *st)
 
     if (osize != cell_size)
       {
-        if (0 && !st->warned)
+# if 0
+        if (!st->warned)
           {
             fprintf (stderr,
              "%s: throttling cell size from %d to %d because of %dM limit.\n",
@@ -432,6 +441,7 @@ setup_display (struct state *st)
                      ((float) alloc_size) / (1 << 20));
             st->warned = 1;
           }
+# endif
       }
 
     st->xSize = st->arr_width ? st->windowWidth / st->arr_width : 0;
