@@ -396,7 +396,7 @@ fizzle (eraser_state *st)
       unsigned int i;
 
       st->fizzle_rnd =
-        (unsigned short *) malloc (sizeof(unsigned short) * chunks);
+        (unsigned short *) calloc (sizeof(unsigned short), chunks);
 
       if (! st->fizzle_rnd)
         return;
@@ -548,7 +548,7 @@ random_squares (eraser_state *st)
 
 /* I first saw something like this, albeit in reverse, in an early Tetris
    implementation for the Mac.
-    -- Torbjrn Andersson <torbjorn@dev.eurotime.se>
+    -- Torbjörn Andersson <torbjorn@dev.eurotime.se>
  */
 static void
 slide_lines (eraser_state *st)
